

def getUsernameFromGraphName(graphName):
	rs = graphName
	if "carre.kmi.open.ac.uk/users/" in graphName:
		rs = rs.replace("http://carre.kmi.open.ac.uk/users/","")
		rs = rs.replace("https://carre.kmi.open.ac.uk/users/","")
	return rs

def getConnections(manufacturer=None):
	from SPARQLWrapper import SPARQLWrapper, DIGEST, JSON, XML, TURTLE, N3, RDF
	import json

	sparql = SPARQLWrapper("http://carre.kmi.open.ac.uk:8890/sparql-auth")
	sparql.setHTTPAuth(DIGEST)
	sparql.setCredentials("INSERT_ROOT_USERNAME", "INSERT_ROOT_PASSWORD")
	if manufacturer is None:
		sql = """
			SELECT DISTINCT ?g as ?graphName ?s as ?manufacturer ?o as ?date ?o2 as ?firstConnected
			WHERE {
			  GRAPH ?g {
			    ?s <http://carre.kmi.open.ac.uk/ontology/sensors.owl#last_measured> ?o .
			    ?s <http://carre.kmi.open.ac.uk/ontology/sensors.owl#first_connected> ?o2 .
			    FILTER(?o2>?o)
			  }
			}
		  """
	else:
		manufacturer = "http://carre.kmi.open.ac.uk/manufacturers/" + manufacturer
		sql = """
			SELECT DISTINCT ?g as ?graphName  <""" + manufacturer+"""> as ?manufacturer ?o as ?date ?o2 as ?firstConnected
			WHERE {
			  GRAPH ?g {
			    <"""+manufacturer+"""> <http://carre.kmi.open.ac.uk/ontology/sensors.owl#last_measured> ?o .
			    <"""+manufacturer+"""> <http://carre.kmi.open.ac.uk/ontology/sensors.owl#first_connected> ?o2 .
			    FILTER(?o2>?o)
			  }
			}
		  """
	# print sql


	sparql.setQuery(sql)

	sparql.setReturnFormat(JSON)
	results = sparql.query().convert()
	services = results['results']['bindings']
	return services

def getUserId(manufacturer, graphName):
	from SPARQLWrapper import SPARQLWrapper, DIGEST, JSON, XML, TURTLE, N3, RDF
	import json

	sparql = SPARQLWrapper("http://carre.kmi.open.ac.uk:8890/sparql-auth")
	sparql.setHTTPAuth(DIGEST)
	sparql.setCredentials("INSERT_ROOT_USERNAME", "INSERT_ROOT_PASSWORD")


	sql = """
			SELECT ?object2 as ?userId FROM <"""+ graphName + """> WHERE

			{ ?subject ?predicate <http://carre.kmi.open.ac.uk/manufacturers/""" + manufacturer + """>.
			?subject <http://carre.kmi.open.ac.uk/ontology/sensors.owl#has_user_id> ?object2}
	"""
	sparql.setQuery(sql)

	sparql.setReturnFormat(JSON)
	results = sparql.query().convert()
	results = results['results']['bindings']
	if len(results)==1:
		rs = results[0]['userId']['value']
	else:
		rs = ""
	return rs

def updateLastMeasured(graphName, provider, date):
	from SPARQLWrapper import SPARQLWrapper, DIGEST, JSON, XML, TURTLE, N3, RDF, SELECT
	import json
	import datetime
	
	newDate = date + datetime.timedelta(days=1)
	sparql = SPARQLWrapper(endpoint="http://carre.kmi.open.ac.uk:8890/sparql-auth", updateEndpoint="http://carre.kmi.open.ac.uk:8890/sparql-auth")
	sparql.setHTTPAuth(DIGEST)
	sparql.setCredentials("INSERT_ROOT_USERNAME", "INSERT_ROOT_PASSWORD")
	manufacturer = "http://carre.kmi.open.ac.uk/manufacturers/" + provider

	sql = "WITH <" + graphName + "> DELETE { <"+manufacturer+"> <http://carre.kmi.open.ac.uk/ontology/sensors.owl#last_measured> ?object }  WHERE { ?subject ?predicate ?object . <" + manufacturer + "> <http://carre.kmi.open.ac.uk/ontology/sensors.owl#last_measured> ?object}"

	sparql.setQuery(sql)
	sparql.queryType= SELECT
	rs = sparql.query()

	sql = "INSERT IN <" + graphName + "> { <" + manufacturer + "> <http://carre.kmi.open.ac.uk/ontology/sensors.owl#last_measured> \"" + newDate.strftime("%Y-%m-%d") + "\"^^xsd:date}"
	sparql.setQuery(sql)
	sparql.queryType= SELECT
	rs = sparql.query()

	return rs

def updateFitbit():
	import requests
	import json
	import datetime

	connections = getConnections("fitbit")
	rs = 0
	for connection in connections:
		date = connection['date']['value']
		d = datetime.datetime.strptime(date, "%Y-%m-%d").date()
		graphName = connection['graphName']['value']
		manufacturer = connection['manufacturer']['value']

		username = getUsernameFromGraphName(graphName)
		ownerId = getUserId("fitbit",graphName)
		if ownerId=="":
			pass
		data = {}
		data['date'] = date
		data['ownerId'] = ownerId
		data['ownerType'] = "user"
		data['subscriptionId'] = username
		data = [data]
		rs = requests.post("https://carre.kmi.open.ac.uk/tomcat/responder/fitbit",data=json.dumps(data))#,verify=False)
		updateLastMeasured(graphName,"fitbit", d)
	return rs

def updateMisfit():
	import requests
	import datetime
	now = datetime.datetime.now().date()
	connections = getConnections("misfit")
	for connection in connections:
		date = connection['date']['value']
		d = datetime.datetime.strptime(date, "%Y-%m-%d").date()
		date_str = date + " 00:00:00 UTC"

		
		if (now-d).days>30:
			date_to = d+datetime.timedelta(30)
			date_to_str = date_to.strftime("%Y-%m-%d")
		else:
			date_to_str = connection['firstConnected']['value']
		
		date_to_str = date_to_str + " 00:00:00 UTC"
		date_to = date_to_str.replace(" 00:00:00 UTC", "")
		date_to = datetime.datetime.strptime(date_to, "%Y-%m-%d").date()

		graphName = connection['graphName']['value']
		manufacturer = connection['manufacturer']['value']
		ownerId = getUserId("misfit",graphName)

		data = {}
		data["Type"] = "Notification"
		message = {}
		message["type"] = "sleeps"
		message["action"] = "created"
		message["ownerId"] = ownerId
		message["updatedAt"] = date_to_str
		message["dataFrom"] = date_str
		
		sessions = {}
		sessions["type"] = "sessions"
		sessions["action"] = "updated"
		sessions["ownerId"] = ownerId
		sessions["updatedAt"] = date_to_str
		sessions["dataFrom"] = date_str
		
		data["Message"] = [message, sessions]
		
		url = "https://carre.kmi.open.ac.uk/tomcat/responder/misfit"
		import urllib2
		import json
		data_str = json.dumps(data)
		rs = urllib2.urlopen(url, data_str)
		

		updateLastMeasured(graphName, "misfit", date_to)
	return 1

def updateWithings():
	import requests
	import datetime
	connections = getConnections("withings")
	for connection in connections:
		date = connection['date']['value']
		d = datetime.datetime.now()
		graphName = connection['graphName']['value']
		manufacturer = connection['manufacturer']['value']
		ownerId = getUserId("withings",graphName)

		date= datetime.datetime(2013,11,1).strftime("%s")
		data = {}
		data['startdate'] = date
		data['enddate'] = datetime.datetime.now().strftime("%s")
		data['userid'] = ownerId
		rs = requests.post("https://carre.kmi.open.ac.uk/tomcat/responder/withings", data=data,verify=False)
		updateLastMeasured(graphName, "withings", d)
	return rs

def updateIHealth():
	import requests
	import json
	import datetime
	connections = getConnections("ihealth")
	rs = 0
	for connection in connections:
		date = connection['date']['value']
		d = datetime.datetime.strptime(date, "%Y-%m-%d").date()
		graphName = connection['graphName']['value']
		manufacturer = connection['manufacturer']['value']
		ownerId = getUserId("ihealth",graphName)
		data = {}
		data['CollectionType'] = ""
		data['MDate'] = str(date) + " 00:00:00"
		data['UserID'] = ownerId
		data['SubscriptionId'] = "1234"
		rs = requests.post("https://carre.kmi.open.ac.uk/tomcat/responder/iHealth",data=json.dumps(data),verify=False)
		updateLastMeasured(graphName, "ihealth", d)
	return rs

def updateGoogleFit():
	import requests
	import json
	import datetime
	connections = getConnections("googlefit")
	rs = 0
	for connection in connections:
		date = connection['date']['value']
		d = datetime.datetime.strptime(date, "%Y-%m-%d").date()
		graphName = connection['graphName']['value']
		manufacturer = connection['manufacturer']['value']
		ownerId = getUsernameFromGraphName(graphName)
		date = connection['date']['value']
		date = datetime.datetime.strptime(date, "%Y-%m-%d").date()
		date= date.strftime("%Y-%m-%d %H:%M:%S")
		data = {}
		data['start_date'] = date
		data['end_date'] = (d + datetime.timedelta(days=1)).strftime("%Y-%m-%d %H:%M:%S")
		data['user_id'] = ownerId
		rs = requests.post("https://carre.kmi.open.ac.uk/tomcat/responder/googlefit", data=json.dumps(data),verify=False)
		updateLastMeasured(graphName, "googlefit", d)

	return rs